#version 330 core
#extension GL_ARB_explicit_uniform_location : enable
#extension GL_ARB_separate_shader_objects : enable

layout(location = 0) in vec3 vertPosition;
layout(location = 1) in vec2 vertTexUV;
layout(location = 2) in vec3 vertNormal;
layout(location = 3) in vec3 vertTangent;
layout(location = 4) in vec3 vertBitangent;

out vec3 toEyeDirTS;
out vec2 texCoords;
out vec3 positionVS;
out vec3 positionOS;
out vec3 normalVS;
out vec3 tangentOS;
out vec3 bitangentOS;
out vec3 normalOS;

uniform mat4 model;
uniform mat4 view_proj;
uniform mat4 view;
uniform mat4 projection;

uniform int numLights;

vec3 ReOrthogonalize(vec3 first, vec3 second)
{
	return normalize(first - dot(first, second) * second);
}

void main()
{

	mat4 vmMatrix = view * model;

	// normal
	normalOS = normalize(vertNormal);
	normalVS = normalize( (vmMatrix * vec4(normalOS, 0.0)).xyz );

	// tangents
	tangentOS = ReOrthogonalize(vertTangent, normalOS);
	bitangentOS = cross(normalOS, tangentOS);

	// matrices
	mat3 matrixTStoOS = mat3(tangentOS, bitangentOS, normalOS);
	mat3 matrixOStoTS = transpose(matrixTStoOS);
	mat3 matrixVStoTS = matrixOStoTS * mat3(inverse(vmMatrix));
	
	// positions
	positionOS = vertPosition;
	positionVS = (vmMatrix * vec4(positionOS, 1.0)).xyz;

	// eye
	toEyeDirTS = matrixVStoTS * normalize(vec3(0, 0, 0) - positionVS.xyz);
	
	// UV
	texCoords = vec2(vertTexUV.x, -vertTexUV.y);

	gl_Position = projection * vec4(positionVS, 1.0);
}